@@ -1,6 +1,6 @@
 //
 //  ========================================================================
-//  Copyright (c) 1995-2020 Mort Bay Consulting Pty Ltd and others.
+//  Copyright (c) 1995-2022 Mort Bay Consulting Pty Ltd and others.
 //  ------------------------------------------------------------------------
 //  All rights reserved. This program and the accompanying materials
 //  are made available under the terms of the Eclipse Public License v1.0
@@ -29,8 +29,10 @@ import java.util.List;
 import java.util.ListIterator;
 import java.util.Map;
 import java.util.NoSuchElementException;
+import java.util.Objects;
 import java.util.Set;
 import java.util.StringTokenizer;
+import java.util.function.BiFunction;
 import java.util.function.ToIntFunction;
 import java.util.stream.Stream;
 
@@ -87,6 +89,152 @@ public class HttpFields implements Iterable<HttpField>
         _size = fields._size;
     }
 
+    /**
+     * <p>Computes a single field for the given HttpHeader and for existing fields with the same header.</p>
+     *
+     * <p>The compute function receives the field name and a list of fields with the same name
+     * so that their values can be used to compute the value of the field that is returned
+     * by the compute function.
+     * If the compute function returns {@code null}, the fields with the given name are removed.</p>
+     * <p>This method comes handy when you want to add an HTTP header if it does not exist,
+     * or add a value if the HTTP header already exists, similarly to
+     * {@link Map#compute(Object, BiFunction)}.</p>
+     *
+     * <p>This method can be used to {@link #put(HttpField) put} a new field (or blindly replace its value):</p>
+     * <pre>
+     * httpFields.computeField("X-New-Header",
+     *     (name, fields) -&gt; new HttpField(name, "NewValue"));
+     * </pre>
+     *
+     * <p>This method can be used to coalesce many fields into one:</p>
+     * <pre>
+     * // Input:
+     * GET / HTTP/1.1
+     * Host: localhost
+     * Cookie: foo=1
+     * Cookie: bar=2,baz=3
+     * User-Agent: Jetty
+     *
+     * // Computation:
+     * httpFields.computeField("Cookie", (name, fields) -&gt;
+     * {
+     *     // No cookies, nothing to do.
+     *     if (fields == null)
+     *         return null;
+     *
+     *     // Coalesces all cookies.
+     *     String coalesced = fields.stream()
+     *         .flatMap(field -&gt; Stream.of(field.getValues()))
+     *         .collect(Collectors.joining(", "));
+     *
+     *     // Returns a single Cookie header with all cookies.
+     *     return new HttpField(name, coalesced);
+     * }
+     *
+     * // Output:
+     * GET / HTTP/1.1
+     * Host: localhost
+     * Cookie: foo=1, bar=2, baz=3
+     * User-Agent: Jetty
+     * </pre>
+     *
+     * <p>This method can be used to replace a field:</p>
+     * <pre>
+     * httpFields.computeField("X-Length", (name, fields) -&gt;
+     * {
+     *     if (fields == null)
+     *         return null;
+     *
+     *     // Get any value among the X-Length headers.
+     *     String length = fields.stream()
+     *         .map(HttpField::getValue)
+     *         .findAny()
+     *         .orElse("0");
+     *
+     *     // Replace X-Length headers with X-Capacity header.
+     *     return new HttpField("X-Capacity", length);
+     * });
+     * </pre>
+     *
+     * <p>This method can be used to remove a field:</p>
+     * <pre>
+     * httpFields.computeField("Connection", (name, fields) -&gt; null);
+     * </pre>
+     *
+     * @param header the HTTP header
+     * @param computeFn the compute function
+     */
+    public void computeField(HttpHeader header, BiFunction<HttpHeader, List<HttpField>, HttpField> computeFn)
+    {
+        computeField(header, computeFn, (f, h) -> f.getHeader() == h);
+    }
+
+    /**
+     * <p>Computes a single field for the given HTTP header name and for existing fields with the same name.</p>
+     *
+     * @param name the HTTP header name
+     * @param computeFn the compute function
+     * @see #computeField(HttpHeader, BiFunction)
+     */
+    public void computeField(String name, BiFunction<String, List<HttpField>, HttpField> computeFn)
+    {
+        computeField(name, computeFn, HttpField::is);
+    }
+
+    private <T> void computeField(T header, BiFunction<T, List<HttpField>, HttpField> computeFn, BiFunction<HttpField, T, Boolean> matcher)
+    {
+        // Look for first occurrence
+        int first = -1;
+        for (int i = 0; i < _size; i++)
+        {
+            HttpField f = _fields[i];
+            if (matcher.apply(f, header))
+            {
+                first = i;
+                break;
+            }
+        }
+
+        // If the header is not found, add a new one;
+        if (first < 0)
+        {
+            HttpField newField = computeFn.apply(header, null);
+            if (newField != null)
+                add(newField);
+            return;
+        }
+
+        // Are there any more occurrences?
+        List<HttpField> found = null;
+        for (int i = first + 1; i < _size; i++)
+        {
+            HttpField f = _fields[i];
+            if (matcher.apply(f, header))
+            {
+                if (found == null)
+                {
+                    found = new ArrayList<>();
+                    found.add(_fields[first]);
+                }
+                // Remember and remove additional fields
+                found.add(f);
+                remove(i--);
+            }
+        }
+
+        // If no additional fields were found, handle singleton case
+        if (found == null)
+            found = Collections.singletonList(_fields[first]);
+        else
+            found = Collections.unmodifiableList(found);
+
+        HttpField newField = computeFn.apply(header, found);
+        if (newField == null)
+            remove(first);
+        else
+            _fields[first] = newField;
+    }
+
     public int size()
     {
         return _size;
@@ -166,7 +314,7 @@ public class HttpFields implements Iterable<HttpField>
         for (int i = 0; i < _size; i++)
         {
             HttpField f = _fields[i];
-            if (f.getName().equalsIgnoreCase(name))
+            if (f.is(name))
                 return f;
         }
         return null;
@@ -188,6 +336,22 @@ public class HttpFields implements Iterable<HttpField>
         return fields == null ? Collections.emptyList() : fields;
     }
 
+    public List<HttpField> getFields(String name)
+    {
+        List<HttpField> fields = null;
+        for (int i = 0; i < _size; i++)
+        {
+            HttpField f = _fields[i];
+            if (f.is(name))
+            {
+                if (fields == null)
+                    fields = new ArrayList<>();
+                fields.add(f);
+            }
+        }
+        return fields == null ? Collections.emptyList() : fields;
+    }
+
     public boolean contains(HttpField field)
     {
         for (int i = _size; i-- > 0; )
@@ -215,7 +379,7 @@ public class HttpFields implements Iterable<HttpField>
         for (int i = _size; i-- > 0; )
         {
             HttpField f = _fields[i];
-            if (f.getName().equalsIgnoreCase(name) && f.contains(value))
+            if (f.is(name) && f.contains(value))
                 return true;
         }
         return false;
@@ -237,7 +401,7 @@ public class HttpFields implements Iterable<HttpField>
         for (int i = _size; i-- > 0; )
         {
             HttpField f = _fields[i];
-            if (f.getName().equalsIgnoreCase(name))
+            if (f.is(name))
                 return true;
         }
         return false;
@@ -271,7 +435,7 @@ public class HttpFields implements Iterable<HttpField>
         for (int i = 0; i < _size; i++)
         {
             HttpField f = _fields[i];
-            if (f.getName().equalsIgnoreCase(header))
+            if (f.is(header))
                 return f.getValue();
         }
         return null;
@@ -303,14 +467,18 @@ public class HttpFields implements Iterable<HttpField>
      */
     public List<String> getValuesList(String name)
     {
-        final List<String> list = new ArrayList<>();
+        List<String> list = null;
         for (int i = 0; i < _size; i++)
         {
             HttpField f = _fields[i];
-            if (f.getName().equalsIgnoreCase(name))
+            if (f.is(name))
+            {
+                if (list == null)
+                    list = new ArrayList<>(size() - i);
                 list.add(f.getValue());
+            }
         }
-        return list;
+        return list == null ? Collections.emptyList() : list;
     }
 
     /**
@@ -358,7 +526,7 @@ public class HttpFields implements Iterable<HttpField>
         for (int i = 0; i < _size; i++)
         {
             HttpField f = _fields[i];
-            if (f.getName().equalsIgnoreCase(name))
+            if (f.is(name))
             {
                 if (existing == null)
                     existing = new QuotedCSV(false);
@@ -446,7 +614,7 @@ public class HttpFields implements Iterable<HttpField>
         QuotedCSV values = null;
         for (HttpField f : this)
         {
-            if (f.getName().equalsIgnoreCase(name))
+            if (f.is(name))
             {
                 if (values == null)
                     values = new QuotedCSV(keepQuotes);
@@ -504,7 +672,7 @@ public class HttpFields implements Iterable<HttpField>
         QuotedQualityCSV values = null;
         for (HttpField f : this)
         {
-            if (f.getName().equalsIgnoreCase(name))
+            if (f.is(name))
             {
                 if (values == null)
                     values = new QuotedQualityCSV();
@@ -526,7 +694,7 @@ public class HttpFields implements Iterable<HttpField>
         {
             final HttpField f = _fields[i];
 
-            if (f.getName().equalsIgnoreCase(name) && f.getValue() != null)
+            if (f.is(name) && f.getValue() != null)
             {
                 final int first = i;
                 return new Enumeration<String>()
@@ -542,7 +710,7 @@ public class HttpFields implements Iterable<HttpField>
                             while (i < _size)
                             {
                                 field = _fields[i++];
-                                if (field.getName().equalsIgnoreCase(name) && field.getValue() != null)
+                                if (field.is(name) && field.getValue() != null)
                                     return true;
                             }
                             field = null;
@@ -631,7 +799,8 @@ public class HttpFields implements Iterable<HttpField>
             {
                 if (put)
                 {
-                    System.arraycopy(_fields, i + 1, _fields, i, --_size - i);
+                    _size--;
+                    System.arraycopy(_fields, i + 1, _fields, i, _size - i);
                 }
                 else
                 {
@@ -671,6 +840,8 @@ public class HttpFields implements Iterable<HttpField>
      */
     public void put(HttpHeader header, String value)
     {
+        Objects.requireNonNull(header, "header must not be null");
+
         if (value == null)
             remove(header);
         else
@@ -685,7 +856,12 @@ public class HttpFields implements Iterable<HttpField>
      */
     public void put(String name, List<String> list)
     {
+        Objects.requireNonNull(name, "name must not be null");
+
         remove(name);
+        if (list == null)
+            return;
+
         for (String v : list)
         {
             if (v != null)
@@ -711,7 +887,8 @@ public class HttpFields implements Iterable<HttpField>
 
     public void add(HttpHeader header, HttpHeaderValue value)
     {
-        add(header, value.toString());
+        if (value != null)
+            add(header, value.toString());
     }
 
     /**
@@ -723,6 +900,8 @@ public class HttpFields implements Iterable<HttpField>
      */
     public void add(HttpHeader header, String value)
     {
+        Objects.requireNonNull(header, "header must not be null");
+
         if (value == null)
             throw new IllegalArgumentException("null value");
 
@@ -745,7 +924,7 @@ public class HttpFields implements Iterable<HttpField>
             if (f.getHeader() == name)
             {
                 removed = f;
-                System.arraycopy(_fields, i + 1, _fields, i, --_size - i);
+                remove(i);
             }
         }
         return removed;
@@ -763,15 +942,22 @@ public class HttpFields implements Iterable<HttpField>
         for (int i = _size; i-- > 0; )
         {
             HttpField f = _fields[i];
-            if (f.getName().equalsIgnoreCase(name))
+            if (f.is(name))
             {
                 removed = f;
-                System.arraycopy(_fields, i + 1, _fields, i, --_size - i);
+                remove(i);
             }
         }
         return removed;
     }
 
+    private void remove(int i)
+    {
+        _size--;
+        System.arraycopy(_fields, i + 1, _fields, i, _size - i);
+        _fields[_size] = null;
+    }
+
     /**
      * Get a header as an long value. Returns the value of an integer field or -1 if not found. The
      * case of the field name is ignored.
@@ -966,8 +1152,11 @@ public class HttpFields implements Iterable<HttpField>
      *
      * @param fields the fields to add
      */
+    @Deprecated
     public void add(HttpFields fields)
     {
+        // TODO this implementation doesn't do what the javadoc says and is really the same
+        // as addAll, which is renamed to add anyway in 10.
         if (fields == null)
             return;
 
@@ -1160,8 +1349,7 @@ public class HttpFields implements Iterable<HttpField>
         {
             if (_current < 0)
                 throw new IllegalStateException();
-            _size--;
-            System.arraycopy(_fields, _current + 1, _fields, _current, _size - _current);
+            HttpFields.this.remove(_current);
             _fields[_size] = null;
             _cursor = _current;
             _current = -1;
@@ -1185,7 +1373,7 @@ public class HttpFields implements Iterable<HttpField>
         @Override
         public int nextIndex()
         {
-            return _cursor + 1;
+            return _cursor;
         }
 
         @Override
@@ -1199,16 +1387,23 @@ public class HttpFields implements Iterable<HttpField>
         {
             if (_current < 0)
                 throw new IllegalStateException();
-            _fields[_current] = field;
+            if (field == null)
+                remove();
+            else
+                _fields[_current] = field;
         }
 
         @Override
         public void add(HttpField field)
         {
-            _fields = Arrays.copyOf(_fields, _fields.length + 1);
-            System.arraycopy(_fields, _cursor, _fields, _cursor + 1, _size++);
-            _fields[_cursor++] = field;
-            _current = -1;
+            if (field != null)
+            {
+                _fields = Arrays.copyOf(_fields, _fields.length + 1);
+                System.arraycopy(_fields, _cursor, _fields, _cursor + 1, _size - _cursor);
+                _fields[_cursor++] = field;
+                _size++;
+                _current = -1;
+            }
         }
     }
 }
