@@ -1,6 +1,6 @@
 //
 //  ========================================================================
-//  Copyright (c) 1995-2021 Mort Bay Consulting Pty Ltd and others.
+//  Copyright (c) 1995-2022 Mort Bay Consulting Pty Ltd and others.
 //  ------------------------------------------------------------------------
 //  All rights reserved. This program and the accompanying materials
 //  are made available under the terms of the Eclipse Public License v1.0
@@ -67,6 +67,7 @@ public class HTTP2Stream extends IdleTimeout implements IStream, Callback, Dumpa
     private boolean remoteReset;
     private Listener listener;
     private long dataLength;
+    private boolean committed;
 
     public HTTP2Stream(Scheduler scheduler, ISession session, int streamId, boolean local)
     {
@@ -136,14 +137,23 @@ public class HTTP2Stream extends IdleTimeout implements IStream, Callback, Dumpa
     @Override
     public void reset(ResetFrame frame, Callback callback)
     {
+        Throwable resetFailure = null;
         synchronized (this)
         {
             if (isReset())
-                return;
-            localReset = true;
-            failure = new EOFException("reset");
+            {
+                resetFailure = failure;
+            }
+            else
+            {
+                localReset = true;
+                failure = new EOFException("reset");
+            }
         }
-        ((HTTP2Session)session).reset(this, frame, callback);
+        if (resetFailure != null)
+            callback.failed(resetFailure);
+        else
+            ((HTTP2Session)session).reset(this, frame, callback);
     }
 
     private boolean startWrite(Callback callback)
@@ -226,6 +236,18 @@ public class HTTP2Stream extends IdleTimeout implements IStream, Callback, Dumpa
         return closeState.get() == CloseState.LOCALLY_CLOSED;
     }
 
+    @Override
+    public void commit()
+    {
+        committed = true;
+    }
+
+    @Override
+    public boolean isCommitted()
+    {
+        return committed;
+    }
+
     @Override
     public boolean isOpen()
     {
@@ -278,6 +300,11 @@ public class HTTP2Stream extends IdleTimeout implements IStream, Callback, Dumpa
         notIdle();
         switch (frame.getType())
         {
+            case PREFACE:
+            {
+                onNewStream(callback);
+                break;
+            }
             case HEADERS:
             {
                 onHeaders((HeadersFrame)frame, callback);
@@ -315,6 +342,12 @@ public class HTTP2Stream extends IdleTimeout implements IStream, Callback, Dumpa
         }
     }
 
+    private void onNewStream(Callback callback)
+    {
+        notifyNewStream(this);
+        callback.succeeded();
+    }
+
     private void onHeaders(HeadersFrame frame, Callback callback)
     {
         MetaData metaData = frame.getMetaData();
@@ -357,10 +390,10 @@ public class HTTP2Stream extends IdleTimeout implements IStream, Callback, Dumpa
             }
         }
 
-        if (updateClose(frame.isEndStream(), CloseState.Event.RECEIVED))
-            session.removeStream(this);
-
+        boolean closed = updateClose(frame.isEndStream(), CloseState.Event.RECEIVED);
         notifyData(this, frame, callback);
+        if (closed)
+            session.removeStream(this);
     }
 
     private void onReset(ResetFrame frame, Callback callback)
@@ -373,6 +406,8 @@ public class HTTP2Stream extends IdleTimeout implements IStream, Callback, Dumpa
         close();
         if (session.removeStream(this))
             notifyReset(this, frame, callback);
+        else
+            callback.succeeded();
     }
 
     private void onPush(PushPromiseFrame frame, Callback callback)
@@ -397,6 +432,8 @@ public class HTTP2Stream extends IdleTimeout implements IStream, Callback, Dumpa
         close();
         if (session.removeStream(this))
             notifyFailure(this, frame, callback);
+        else
+            callback.succeeded();
     }
 
     @Override
@@ -576,6 +613,15 @@ public class HTTP2Stream extends IdleTimeout implements IStream, Callback, Dumpa
             callback.failed(x);
     }
 
+    @Override
+    public InvocationType getInvocationType()
+    {
+        synchronized (this)
+        {
+            return sendCallback != null ? sendCallback.getInvocationType() : Callback.super.getInvocationType();
+        }
+    }
+
     private Callback endWrite()
     {
         synchronized (this)
@@ -586,6 +632,22 @@ public class HTTP2Stream extends IdleTimeout implements IStream, Callback, Dumpa
         }
     }
 
+    private void notifyNewStream(Stream stream)
+    {
+        Listener listener = this.listener;
+        if (listener != null)
+        {
+            try
+            {
+                listener.onNewStream(stream);
+            }
+            catch (Throwable x)
+            {
+                LOG.info("Failure while notifying listener {}", listener, x);
+            }
+        }
+    }
+
     private void notifyData(Stream stream, DataFrame frame, Callback callback)
     {
         Listener listener = this.listener;
